import * as React from "react";
import { ActionProviderProps, createActionProvider } from "src/core/actionproviders/createActionProvider";
import * as Models from "src/core/models";
import { CourseActions } from "src/core/store/course";

export type FetchCourseProps = ActionProviderProps<CourseActions.FetchCourse> & {
	autoFetch?: boolean;
	course: string | Models.Course;
};

export const {
	ActionProvider: FetchCourse,
	ActionContextProvider: FetchCourseContextProvider,
	ActionContextConsumer: FetchCourseContextConsumer
} = createActionProvider<CourseActions.FetchCourse, FetchCourseProps>({
	onUnmount: () => {
		/**/
	},
	createAction: (component, rootStore) => {
		const { course, autoFetch } = component.props;
		const fetchCourse = new CourseActions.FetchCourse(rootStore.courseStore);
		fetchCourse.setCourse(course);

		if ((course !== undefined && autoFetch) || autoFetch === undefined) fetchCourse.fetch();

		return fetchCourse;
	},
	onPropsChange: (component, nextProps) => {
		const { action } = component;
		const { course, autoFetch } = nextProps;

		if (component.props.course !== course) {
			action.setCourse(course);

			if ((course !== undefined && autoFetch) || autoFetch === undefined) action.fetch();
		}
	}
});
