import * as React from "react";
import { ActionProviderProps, createActionProvider } from "src/core/actionproviders/createActionProvider";
import * as Models from "src/core/models";
import { CourseClassActions } from "src/core/store/courseclass";

export type CreateCourseClassProps = ActionProviderProps<CourseClassActions.CreateCourseClass> & {
	courseCode: string;
	course: Models.Course | undefined;
};

export const {
	ActionProvider: CreateCourseClassProvider,
	ActionContextProvider: CreateCourseClassContextProvider,
	ActionContextConsumer: CreateCourseClassContextConsumer
} = createActionProvider<CourseClassActions.CreateCourseClass, CreateCourseClassProps>({
	onUnmount: () => {
		/**/
	},
	createAction: (component, rootStore) => {
		const { course } = component.props;
		const action = new CourseClassActions.CreateCourseClass(rootStore.courseClassStore);

		if (course) action.setCourse(course);

		return action;
	},
	onPropsChange: (component, nextProps) => {
		const { props, action } = component;

		if (props.course !== nextProps.course && nextProps.course) action.setCourse(nextProps.course);
	}
});
