import * as React from "react";
import { ActionProviderProps, createActionProvider } from "src/core/actionproviders/createActionProvider";
import { CourseClassActions } from "src/core/store/courseclass";

export type FetchCourseClassProps = ActionProviderProps<CourseClassActions.FetchCourseClass> & {
	courseCode: string;
	classNo: string | number;
	autoFetch?: boolean;
};

export const {
	ActionProvider: FetchCourseClass,
	ActionContextProvider: FetchCourseClassContextProvider,
	ActionContextConsumer: FetchCourseClassContextConsumer
} = createActionProvider<CourseClassActions.FetchCourseClass, FetchCourseClassProps>({
	onUnmount: () => {
		/**/
	},
	createAction: (component, rootStore) => {
		const { props } = component;
		const fetchCourse = new CourseClassActions.FetchCourseClass(rootStore.courseClassStore);

		fetchCourse.setData(component.props);

		if (props.autoFetch || props.autoFetch === undefined) fetchCourse.fetch();
		return fetchCourse;
	},
	onPropsChange: (component, nextProps) => {
		const { action, props } = component;

		if (props.courseCode !== nextProps.courseCode || props.classNo !== nextProps.classNo) {
			action.setData(nextProps);

			if (nextProps.autoFetch || nextProps.autoFetch === undefined) action.fetch();
		}
	}
});
