import * as React from "react";
import { ActionProviderProps, createActionProvider } from "src/core/actionproviders/createActionProvider";
import * as Models from "src/core/models";
import { FAQActions } from "src/core/store/faq";

export type DeleteFAQProps = ActionProviderProps<FAQActions.DeleteFAQ> & {
	faq: Models.FAQ | string | number;
	autoDelete?: boolean;
};

export const {
	ActionProvider: DeleteFAQ,
	ActionContextProvider: DeleteFAQContextProvider,
	ActionContextConsumer: DeleteFAQContextConsumer
} = createActionProvider<FAQActions.DeleteFAQ, DeleteFAQProps>({
	onUnmount: () => {
		/**/
	},
	createAction: (component, rootStore) => {
		const deleteFAQ = new FAQActions.DeleteFAQ(rootStore.faqStore);

		const { faq } = component.props;

		if (faq !== undefined) deleteFAQ.setFAQ(faq);

		return deleteFAQ;
	},
	onPropsChange: (component, nextProps) => {
		const { action, props } = component;
		const { faq, autoDelete } = nextProps;

		if (props.faq !== faq) {
			action.setFAQ(faq);

			if (autoDelete) action.delete();
		}
	}
});
