import * as React from "react";
import { ActionProviderProps, createActionProvider } from "src/core/actionproviders/createActionProvider";
import * as Models from "src/core/models";
import { FAQActions } from "src/core/store/faq";

export type FetchFAQProps = ActionProviderProps<FAQActions.FetchFAQ> & {
	autoFetch?: boolean;
	faq: string | number | Models.FAQ;
};

export const {
	ActionProvider: FetchFAQ,
	ActionContextProvider: FetchFAQContextProvider,
	ActionContextConsumer: FetchFAQContextConsumer
} = createActionProvider<FAQActions.FetchFAQ, FetchFAQProps>({
	onUnmount: () => {
		/**/
	},
	createAction: (component, rootStore) => {
		const { faq, autoFetch } = component.props;
		const fetchFAQ = new FAQActions.FetchFAQ(rootStore.faqStore);

		fetchFAQ.setFAQ(faq);

		if ((autoFetch || autoFetch === undefined) && fetchFAQ.isReady) fetchFAQ.fetch();

		return fetchFAQ;
	},
	onPropsChange: (component, nextProps) => {
		const { action } = component;
		const { faq, autoFetch } = nextProps;

		if (component.props.faq !== faq) {
			action.setFAQ(faq);

			if ((autoFetch || autoFetch === undefined) && action.isReady) action.fetch();
		}
	}
});
