import * as _ from "lodash";
import { action } from "mobx";
import * as moment from "moment";
import * as Models from "../models";
import { Base, CourseClass, CourseClassJSON, User, UserJSON } from "./internal";

export type CourseJSON = {
	id: number;
	code: string;
	name?: string;
	eva?: string;
	semester?: number;
	year?: number;
	classes?: CourseClassJSON[];
	iconURL?: string;

	createdAt?: string;
	createdBy?: UserJSON;
	updatedAt?: string;
	updatedBy?: UserJSON;
	deletedAt?: string;
	deletedBy?: UserJSON;
}

export class Course extends Base {
	@action
	public static fromJSON(json: CourseJSON): Models.Course {
		const course = new Models.Course();

		this.assignValues(course, json);
		if (_.isArray(json.classes))
			course.classes = json.classes.map(jsonCourseClass => CourseClass.fromJSON(jsonCourseClass));

		if (!_.isNil(json.createdAt)) course.createdAt = moment(json.createdAt);
		if (!_.isNil(json.createdBy)) course.createdBy = User.fromJSON(json.createdBy);

			if (!_.isNil(json.updatedAt)) course.updatedAt = moment(json.updatedAt);
		if (!_.isNil(json.updatedBy)) course.updatedBy = User.fromJSON(json.updatedBy);

		if (!_.isNil(json.deletedAt)) course.deletedAt = moment(json.deletedAt);
		if (!_.isNil(json.deletedBy)) course.deletedBy = User.fromJSON(json.deletedBy);

		return course;
	}
}
