import * as _ from "lodash";
import { action } from "mobx";
import * as moment from "moment";
import * as Models from "../models";
import { Base, Course, CourseJSON, User, UserJSON } from "./internal";

export type CourseClassJSON = {
	id: number;
	number?: number;
	title?: string;
	videoURL?: string;
	disabled?: boolean;
	course?: CourseJSON;

	createdAt?: string;
	createdBy?: UserJSON;
	updatedAt?: string;
	updatedBy?: UserJSON;
	deletedAt?: string;
	deletedBy?: UserJSON;
};

export class CourseClass extends Base {
	@action
	public static fromJSON(json: CourseClassJSON): Models.CourseClass {
		const courseClass = new Models.CourseClass();

		this.assignValues(courseClass, json);
		if (!_.isNil(json.course)) courseClass.course = Course.fromJSON(json.course);

		if (!_.isNil(json.createdAt)) courseClass.createdAt = moment(json.createdAt);
		if (!_.isNil(json.createdBy)) courseClass.createdBy = User.fromJSON(json.createdBy);

		if (!_.isNil(json.updatedAt)) courseClass.updatedAt = moment(json.updatedAt);
		if (!_.isNil(json.updatedBy)) courseClass.updatedBy = User.fromJSON(json.updatedBy);

		if (!_.isNil(json.deletedAt)) courseClass.deletedAt = moment(json.deletedAt);
		if (!_.isNil(json.deletedBy)) courseClass.deletedBy = User.fromJSON(json.deletedBy);

		return courseClass;
	}
}
