import * as _ from "lodash";
import { action } from "mobx";
import * as moment from "moment";
import * as Models from "../models";
import { Base } from "./Base";
import { User, UserJSON } from "./User";

export type FAQJSON = {
	id?: number;
	title?: string;
	content?: string;
	isHTML?: boolean;

	createdAt?: string;
	createdBy?: UserJSON;
	updatedAt?: string;
	updatedBy?: UserJSON;
	deletedAt?: string;
	deletedBy?: UserJSON;
}

export class FAQ extends Base {
	@action
	public static fromJSON(json: FAQJSON): Models.FAQ {
		const faq = new Models.FAQ();
		this.assignValues(faq, json);

		if (!_.isNil(json.createdAt)) faq.createdAt = moment(json.createdAt);
		if (!_.isNil(json.createdBy)) faq.createdBy = User.fromJSON(json.createdBy);

		if (!_.isNil(json.updatedAt)) faq.updatedAt = moment(json.updatedAt);
		if (!_.isNil(json.updatedBy)) faq.updatedBy = User.fromJSON(json.updatedBy);

		if (!_.isNil(json.deletedAt)) faq.deletedAt = moment(json.deletedAt);
		if (!_.isNil(json.deletedBy)) faq.deletedBy = User.fromJSON(json.deletedBy);

		return faq;
	}
}
