import * as _ from "lodash";
import { action } from "mobx";
import * as moment from "moment";
import * as Models from "../models";
import { Base } from "./internal";

export type UserJSON = {
	id?: number;
	email?: string;
	name?: string;

	role?: string;

	createdAt?: string;
	updatedAt?: string;
	deletedAt?: string;
};

export class User extends Base {
	@action
	public static fromJSON(json: UserJSON): Models.User {
		const user = new Models.User();

		this.assignValues(user, json);

		if (!_.isNil(json.createdAt)) user.createdAt = moment(json.createdAt);
		if (!_.isNil(json.updatedAt)) user.updatedAt = moment(json.updatedAt);
		if (!_.isNil(json.deletedAt)) user.deletedAt = moment(json.deletedAt);

		const userRoleKey = _.isNil(json.role)
			? undefined
			: Object.keys(Models.UserRole).find(role => Models.UserRole[role].toLowerCase() === json.role);
		user.role = userRoleKey === undefined ? undefined : Models.UserRole[userRoleKey];

		return user;
	}
}
