import { action, ObservableMap, runInAction } from "mobx";
import * as Models from "../../models";
import { Base } from "../Base";

export class CourseStore extends Base {
	public courses = new ObservableMap<number, Models.Course>();
	public coursesByCode = new ObservableMap<string, number>();

	@action
	public saveCourse(course: Models.Course): Models.Course {
		const { courses, coursesByCode } = this;
		const { courseClassStore, userStore } = this.store;

		let savedCourse = courses.get(course.id);

		if (savedCourse === course) return course;

		if (savedCourse === undefined) {
			courses.set(course.id, course);
			savedCourse = courses.get(course.id)!;
		}

		// Reconciliation
		const { classes } = course;
		if (classes)
			classes.forEach(cc => {
				runInAction(() => {
					cc.course = undefined;
					courseClassStore.saveCourseClass(cc);
					cc.course = savedCourse;
				});
			});

		if (course.createdBy) course.createdBy = userStore.saveUser(course.createdBy);

		if (course.updatedBy) course.updatedBy = userStore.saveUser(course.updatedBy);

		if (course.deletedBy) course.deletedBy = userStore.saveUser(course.deletedBy);

		Object.keys(course).forEach(key => {
			runInAction(() => {
				const value = course[key];

				if (value !== undefined) savedCourse![key] = value;
			});
		});

		if (savedCourse.code !== undefined) coursesByCode.set(savedCourse.code, savedCourse.id);

		return savedCourse;
	}

	@action
	public deleteCourse(course: Models.Course | number | string): boolean {
		const { courses, coursesByCode } = this;

		let courseId: number | undefined =
			typeof course === "number" ? course : typeof course === "object" ? course.id : undefined;

		if (typeof course === "string") {
			const savedCourseId = coursesByCode.get(course);

			if (savedCourseId) courseId = savedCourseId;
		}

		const savedCourse = courseId === undefined ? undefined : courses.get(courseId);

		if (!savedCourse) return false;

		courses.delete(savedCourse.id);
		coursesByCode.delete(savedCourse.code);

		return true;
	}
}
