import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as Models from "../../../models";
import { Base } from "./Base";

export type FilterCoursesState = {
	searchInput: string;
};

export class FilterCourses extends Base {
	public initialState: FilterCoursesState = {
		searchInput: ""
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	@computed
	public get filteredList(): Models.Course[] {
		const { state, courses } = this;
		const { searchInput } = state;

		if (courses === undefined) return [];

		return searchInput.length === 0
			? courses
			: courses.filter(course => {
					let courseNameToCompare = this.prepareToCompare(course.name!);

					return searchInput.split("").every(char => {
						const index = courseNameToCompare.indexOf(this.prepareToCompare(char));

						courseNameToCompare =
							index > -1 && index < courseNameToCompare.length
								? courseNameToCompare.substr(index + 1)
								: "";

						return index !== -1;
					});
			  });
	}

	@observable
	protected courses: Models.Course[] | undefined;

	@action.bound
	public setSearchInput(searchInput: string) {
		this.state.searchInput = searchInput;
	}

	@action
	public init(courses: Models.Course[]) {
		this.courses = courses;
	}

	private prepareToCompare(value: string): string {
		return value
			.toLowerCase()
			.replace("á", "a")
			.replace("é", "e")
			.replace("í", "i")
			.replace("ó", "o")
			.replace("ú", "u");
	}
}
