import { action, ObservableMap, runInAction } from "mobx";
import * as Models from "../../models";
import { Base } from "../Base";

export class CourseClassStore extends Base {
	public courseClasses = new ObservableMap<number, Models.CourseClass>();

	@action
	public saveCourseClass(courseClass: Models.CourseClass): Models.CourseClass {
		const { courseClasses } = this;
		const { courseStore, userStore } = this.store;

		let savedCourse = courseClasses.get(courseClass.id);

		if (savedCourse === courseClass) return courseClass;

		if (savedCourse === undefined) {
			courseClasses.set(courseClass.id, courseClass);
			savedCourse = courseClasses.get(courseClass.id)!;
		}

		// Reconciliation
		if (courseClass.course) courseClass.course = courseStore.saveCourse(courseClass.course);

		if (courseClass.createdBy) courseClass.createdBy = userStore.saveUser(courseClass.createdBy);

		if (courseClass.updatedBy) courseClass.updatedBy = userStore.saveUser(courseClass.updatedBy);

		if (courseClass.deletedBy) courseClass.deletedBy = userStore.saveUser(courseClass.deletedBy);

		Object.keys(courseClass).forEach(key => {
			runInAction(() => {
				const value = courseClass[key];

				if (value !== undefined) savedCourse![key] = value;
			});
		});

		return savedCourse;
	}

	@action
	public deleteCourseClass(courseClass: Models.CourseClass | number): boolean {
		const { courseClasses } = this;

		const savedCourseClass = courseClasses.get(typeof courseClass === "number" ? courseClass : courseClass.id);

		if (!savedCourseClass) return false;

		const { course } = savedCourseClass;
		if (course && course.classes) {
			const courseClassIndex = course.classes.findIndex(cc => cc.id === savedCourseClass.id);

			if (courseClassIndex !== -1) course.classes.splice(courseClassIndex, 1);
		}

		return courseClasses.delete(typeof courseClass === "number" ? courseClass : courseClass.id);
	}
}
