import * as _ from "lodash";
import { action, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type InitialFetchLatestCourseClassesStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchLatestCourseClassesStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchLatestCourseClassesStatus = {
	isLoading: false;
	success: true;
	courseClasses: Models.CourseClass[];
};
export type FailedFetchLatestCourseClassesStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};
export type FetchLatestCourseClassesResponse =
	| SuccessfulFetchLatestCourseClassesStatus
	| FailedFetchLatestCourseClassesStatus;
export type FetchLatestCourseClassesStatus =
	| InitialFetchLatestCourseClassesStatus
	| LoadingFetchLatestCourseClassesStatus
	| FetchLatestCourseClassesResponse;

export class FetchLatestCourseClasses extends Base {
	public initialStatus: FetchLatestCourseClassesStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public async fetch(): Promise<FetchLatestCourseClassesResponse> {
		const { courseClassStore } = this;
		this.setStatus({ isLoading: true });

		const apiResponse = await API.CourseClass.fetchLatestCourseClasses();

		let newStatus: FetchLatestCourseClassesResponse;

		if (apiResponse.success)
			newStatus = {
				isLoading: false,
				success: true,
				courseClasses: apiResponse.courseClasses.map(cc => courseClassStore.saveCourseClass(cc))
			};
		else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		this.setStatus(newStatus);

		return newStatus;
	}

	@action
	protected setStatus<T extends FetchLatestCourseClassesStatus>(status: T) {
		this.status = status;
	}
}
