import * as _ from "lodash";
import { action, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type InitialFetchFAQsStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchFAQsStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchFAQsStatus = {
	isLoading: false;
	success: true;
	faqs: Models.FAQ[];
};
export type FailedFetchFAQsStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};

export type FetchFAQsResponse = SuccessfulFetchFAQsStatus | FailedFetchFAQsStatus;
export type FetchFAQsStatus = InitialFetchFAQsStatus | LoadingFetchFAQsStatus | FetchFAQsResponse;

export class FetchFAQs extends Base {
	public initialStatus: FetchFAQsStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public async fetch(): Promise<FetchFAQsResponse> {
		const { faqStore } = this;
		let newStatus: FetchFAQsStatus = { isLoading: true };
		this.setStatus(newStatus);

		const apiResponse = await API.FAQ.fetchFAQs();

		if (apiResponse.success)
			newStatus = {
				isLoading: false,
				success: true,
				faqs: apiResponse.faqs.map(f => faqStore.saveFAQ(f))
			};
		else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		this.setStatus(newStatus);

		return newStatus;
	}

	@action
	protected setStatus<T extends FetchFAQsStatus>(status: T) {
		this.status = status;
	}
}
