import { action, observable, ObservableMap, runInAction } from "mobx";
import * as Models from "../../models";
import { Base } from "../Base";

export class UserStore extends Base {
	public users = new ObservableMap<number, Models.User>();

	@observable
	public token: string | undefined;

	@observable
	public currentUser: Models.User | undefined;

	@action
	public setToken(token: string | undefined) {
		this.token = token;
	}

	@action
	public setCurrentUser(currentUser: Models.User | undefined) {
		this.currentUser = currentUser;
	}

	@action
	public saveUser(user: Models.User): Models.User {
		const { users } = this;

		let savedUser = users.get(user.id);

		if (savedUser === user) return user;

		if (savedUser === undefined) {
			users.set(user.id, user);
			savedUser = users.get(user.id)!;
		}

		// Reconciliation
		Object.keys(user).forEach(key => {
			runInAction(() => {
				const value = user[key];

				if (value !== undefined) savedUser![key] = value;
			});
		});

		return savedUser;
	}
}
